@@ -276,7 +276,7 @@ do_redraw (GtkSourceGutter *gutter)
 static void
 gtk_source_gutter_map (GtkWidget *widget)
 {
-	gtk_widget_set_cursor_from_name (widget, "arrow");
+	gtk_widget_set_cursor_from_name (widget, "default");
 	GTK_WIDGET_CLASS (gtk_source_gutter_parent_class)->map (widget);
 }
 
@@ -822,6 +822,7 @@ gtk_source_gutter_snapshot (GtkWidget   *widget,
 	if (gtk_source_view_get_highlight_current_line (gutter->view) &&
 	    !gtk_text_buffer_get_selection_bounds (buffer, &cur, &sel))
 	{
+		GtkRoot *root;
 		GdkRGBA highlight;
 		guint cursor_line;
 
@@ -829,7 +830,10 @@ gtk_source_gutter_snapshot (GtkWidget   *widget,
 
 		if (cursor_line >= gtk_source_gutter_lines_get_first (gutter->lines) &&
 		    cursor_line <= gtk_source_gutter_lines_get_last (gutter->lines) &&
-		    _gtk_source_view_get_current_line_number_background (gutter->view, &highlight))
+		    _gtk_source_view_get_current_line_number_background (gutter->view, &highlight) &&
+		    (root = gtk_widget_get_root (GTK_WIDGET (gutter->view))) &&
+		    GTK_IS_WINDOW (root) &&
+		    gtk_window_is_active (GTK_WINDOW (root)))
 		{
 			int width = gtk_widget_get_width (widget);
 			int height;
@@ -877,7 +881,7 @@ gtk_source_gutter_snapshot (GtkWidget   *widget,
 	gtk_snapshot_push_clip (snapshot,
 	                        &GRAPHENE_RECT_INIT (0,
 	                                             0,
-						     clip_width,
+                                                     clip_width,
 	                                             gtk_widget_get_height (widget)));
 
 	/* Now let the renderers draw the content for each line. Because
