@@ -30,6 +30,8 @@
 
 #include "config.h"
 
+#define _GNU_SOURCE
+
 #if defined(HAVE_POSIX_MEMALIGN) && !defined(_XOPEN_SOURCE)
 # define _XOPEN_SOURCE 600
 #endif
@@ -55,10 +57,29 @@
 
 #include <glib.h>
 #include <glib/gi18n-lib.h>
+#include <pango/pango.h>
 
 #include "gtksourceutils.h"
 #include "gtksourceutils-private.h"
 
+#ifdef G_OS_WIN32
+# include <windows.h>
+#endif
+
+#ifdef G_OS_WIN32
+# if PANGO_VERSION_CHECK(1, 52, 0)
+#  include <pango/pangowin32.h>
+#  define FONTLOADING_WITH_PANGOWIN32
+# endif
+#endif
+
+#ifdef ENABLE_FONT_CONFIG
+# include <fontconfig/fontconfig.h>
+# include <pango/pangocairo.h>
+# include <pango/pangofc-fontmap.h>
+# define FONTLOADING_WITH_FONTCONFIG
+#endif
+
 /**
  * gtk_source_utils_unescape_search_text:
  * @text: the text to unescape.
@@ -269,6 +290,41 @@ build_file_listing (const gchar *item,
 		return filenames;
 
 	}
+
+	if (g_str_has_prefix (item, "resource://"))
+	{
+		const char *path = item + strlen ("resource://");
+		char **children;
+
+		children = g_resources_enumerate_children (path, 0, NULL);
+
+		if (children != NULL)
+		{
+			for (guint i = 0; children[i] != NULL; i++)
+			{
+				if (g_str_has_suffix (children[i], suffix))
+				{
+					char *full_path = g_build_path ("/", item, children[i], NULL);
+					const char *resource_full_path = full_path + strlen ("resource://");
+					gsize size = 0;
+
+					if (g_resources_get_info (resource_full_path, 0, &size, NULL, NULL) && size > 0)
+					{
+						filenames = g_slist_prepend (filenames, full_path);
+					}
+					else
+					{
+						g_free (full_path);
+					}
+				}
+			}
+		}
+
+		g_strfreev (children);
+
+		return filenames;
+	}
+
 	dir = g_dir_open (item, 0, NULL);
 
 	if (dir == NULL)
@@ -910,3 +966,162 @@ _gtk_source_utils_aligned_free (gpointer data)
 {
 	aligned_free (data);
 }
+
+#ifdef ENABLE_FONT_CONFIG
+static PangoFontMap *
+load_override_font_fc (void)
+{
+	static FcConfig *map_font_config;
+	PangoFontMap *font_map = NULL;
+
+	font_map = pango_cairo_font_map_new_for_font_type (CAIRO_FONT_TYPE_FT);
+
+	if (!font_map)
+	{
+		g_error ("Unable to create new fontmap");
+		return NULL;
+	}
+
+	if (g_once_init_enter (&map_font_config))
+	{
+		char **font_dirs = _gtk_source_utils_get_default_dirs ("fonts");
+		FcConfig *config = FcInitLoadConfigAndFonts ();
+
+		if (font_dirs != NULL)
+		{
+			for (guint i = 0; font_dirs[i]; i++)
+			{
+				char *font_path = g_build_filename (font_dirs[i], "BuilderBlocks.ttf", NULL);
+
+				if (g_file_test (font_path, G_FILE_TEST_IS_REGULAR))
+				{
+#ifdef G_OS_WIN32
+					/* Reformat the path as expected by win32 FontConfig */
+					FcChar8 *win32_path = FcStrCopyFilename ((const FcChar8 *)font_path);
+					FcConfigAppFontAddFile (config, win32_path);
+					FcStrFree (win32_path);
+#else
+					FcConfigAppFontAddFile (config, (const FcChar8 *)font_path);
+#endif
+				}
+
+				g_free (font_path);
+			}
+		}
+
+		g_strfreev (font_dirs);
+
+		g_once_init_leave (&map_font_config, config);
+	}
+
+	g_assert (map_font_config != NULL);
+
+	pango_fc_font_map_set_config (PANGO_FC_FONT_MAP (font_map), map_font_config);
+
+	return font_map;
+}
+#endif
+
+#ifdef FONTLOADING_WITH_PANGOWIN32
+static PangoFontMap *
+load_override_font_win32 (void)
+{
+	PangoFontMap *font_map;
+	char **font_dirs;
+
+	font_map = pango_cairo_font_map_new_for_font_type (CAIRO_FONT_TYPE_WIN32);
+
+	if (!font_map)
+	{
+		g_error ("Unable to create new fontmap");
+		return NULL;
+	}
+
+	font_dirs = _gtk_source_utils_get_default_dirs ("fonts");
+
+	if (font_dirs != NULL)
+	{
+		for (guint i = 0; font_dirs[i]; i++)
+		{
+			char *font_path = g_build_filename (font_dirs[i], "BuilderBlocks.ttf", NULL);
+
+			if (g_file_test (font_path, G_FILE_TEST_IS_REGULAR))
+			{
+				GError *error = NULL;
+
+				pango_win32_font_map_add_font_file (font_map, font_path, &error);
+
+				if (error)
+				{
+					g_warning("Font loading error: %s", error->message);
+					g_clear_error (&error);
+				}
+			}
+
+			g_free (font_path);
+		}
+	}
+
+	g_strfreev (font_dirs);
+
+	return font_map;
+}
+#endif
+
+PangoFontMap *
+_gtk_source_utils_get_builder_blocks (void)
+{
+	static PangoFontMap *builder_blocks_font_map;
+	static gboolean loaded;
+
+	if (!loaded)
+	{
+		loaded = TRUE;
+
+#ifdef FONTLOADING_WITH_PANGOWIN32
+		if (builder_blocks_font_map == NULL)
+		{
+			builder_blocks_font_map = load_override_font_win32 ();
+		}
+#endif
+
+#ifdef FONTLOADING_WITH_FONTCONFIG
+		if (builder_blocks_font_map == NULL)
+		{
+			builder_blocks_font_map = load_override_font_fc ();
+		}
+#endif
+	}
+
+	return builder_blocks_font_map;
+}
+
+gsize
+_gtk_source_utils_strnlen (const char *str,
+			   gsize       maxlen)
+{
+#ifdef HAVE_STRNLEN
+	return strnlen (str, maxlen);
+#else
+	gsize i = 0;
+
+	for (;;)
+	{
+		if (str[i] == 0)
+		{
+			return i;
+		}
+
+		if (i == maxlen)
+		{
+			return maxlen;
+		}
+
+		i++;
+	}
+
+	g_assert_not_reached ();
+
+	return 0;
+#endif
+}
