@@ -103,11 +103,18 @@ _gtk_source_view_assistants_size_allocate (GtkSourceViewAssistants *assistants,
 
 		g_assert (GTK_SOURCE_IS_ASSISTANT (assistant));
 
-		gtk_popover_present (GTK_POPOVER (assistant));
+		if (gtk_widget_get_visible (GTK_WIDGET (assistant)) &&
+		    gtk_native_get_surface (GTK_NATIVE (assistant)) != NULL)
+		{
+			GtkRequisition req;
+
+			gtk_widget_get_preferred_size (GTK_WIDGET (assistant), NULL, &req);
+			gtk_popover_present (GTK_POPOVER (assistant));
+		}
 	}
 }
 
-static gboolean
+gboolean
 _gtk_source_view_assistants_hide_all (GtkSourceViewAssistants *assistants)
 {
 	gboolean ret = FALSE;
@@ -122,7 +129,7 @@ _gtk_source_view_assistants_hide_all (GtkSourceViewAssistants *assistants)
 
 		if (gtk_widget_get_visible (GTK_WIDGET (assistant)))
 		{
-			gtk_popover_popdown (GTK_POPOVER (assistant));
+			gtk_widget_hide (GTK_WIDGET (assistant));
 			ret = TRUE;
 		}
 	}
