@@ -73,10 +73,15 @@ gtk_source_view_snippets_scroll_to_insert (GtkSourceViewSnippets *snippets)
 	GtkTextIter iter;
 	GdkRectangle area;
 	GdkRectangle visible_rect;
+	guint top_margin;
 	double x, y;
 
 	g_assert (snippets != NULL);
 
+	g_object_get (snippets->view,
+		      "top-margin", &top_margin,
+		      NULL);
+
 	mark = gtk_text_buffer_get_insert (GTK_TEXT_BUFFER (snippets->buffer));
 	gtk_text_buffer_get_iter_at_mark (GTK_TEXT_BUFFER (snippets->buffer), &iter, mark);
 	gtk_text_view_get_iter_location (GTK_TEXT_VIEW (snippets->view), &iter, &area);
@@ -97,7 +102,7 @@ gtk_source_view_snippets_scroll_to_insert (GtkSourceViewSnippets *snippets)
 		y = visible_rect.y;
 
 	gtk_adjustment_set_value (gtk_scrollable_get_hadjustment (GTK_SCROLLABLE (snippets->view)), x);
-	gtk_adjustment_set_value (gtk_scrollable_get_vadjustment (GTK_SCROLLABLE (snippets->view)), y);
+	gtk_adjustment_set_value (gtk_scrollable_get_vadjustment (GTK_SCROLLABLE (snippets->view)), y + top_margin);
 
 	gtk_source_view_snippets_update_informative (snippets);
 }
@@ -363,7 +368,7 @@ gtk_source_view_snippets_update_informative (GtkSourceViewSnippets *snippets)
 
 	if (gtk_widget_get_visible (GTK_WIDGET (snippets->informative)))
 	{
-		_gtk_source_assistant_set_needs_position (GTK_SOURCE_ASSISTANT (snippets->informative));
+		_gtk_source_assistant_update_position (GTK_SOURCE_ASSISTANT (snippets->informative));
 	}
 	else
 	{
@@ -593,6 +598,8 @@ _gtk_source_view_snippets_key_pressed (GtkSourceViewSnippets *snippets,
 		 */
 		if (snippet != NULL)
 		{
+			_gtk_source_view_hide_completion (snippets->view);
+
 			if ((state & modifiers) == 0)
 			{
 				if (!_gtk_source_snippet_move_next (snippet))
@@ -629,6 +636,8 @@ _gtk_source_view_snippets_key_pressed (GtkSourceViewSnippets *snippets,
 		    !has_selection &&
 		    gtk_source_view_snippets_try_expand (snippets, &end))
 		{
+			_gtk_source_view_hide_completion (snippets->view);
+
 			gtk_source_view_snippets_scroll_to_insert (snippets);
 
 			ret = GDK_EVENT_STOP;
